<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

define ('THUMB_FILENAME_SEPARATOR','___');
define ('CACHE_DIR_REL', 'cache/thumbs');
define ('CACHE_DIR', IW_BASE_DIR.'cache/thumbs');

define ('IWIMG_JPG', 2);
define ('IWIMG_GIF', 1);
define ('IWIMG_PNG', 3);

class iwImage 
{
	protected $_imageFile = '';
	protected $_image = null;
	protected $_width = 0;
	protected $_height = 0;
	protected $_type = 0;
	
	public function __construct ($filename='')
	{
		if ($filename != '')
        {
            if (!$this->checkImageExtension())
            {            
                return;
            }
            
		    $this->setPath($filename);
            $this->_valid = true;        
        }
	}
	
	public function __destruct ()
	{
		if ($this->_image)
		{
			imagedestroy($this->_image);
		}
	}
	
    /**
    * @desc Indicates whether this object is wrapping a valid
    *       image.
    * @return bool Returns bool if valid, false otherwise.
    */
    public function isValid()
    {
        if (!$this->_image)
        {
            $this->prepareImage();
        }
        
        return ($this->_image != null);
    }
    
    /**
    * @desc Determines if the image extension for PHP is installed
    * @return bool Returns true if installed, false otherwise.
    */
    public function checkImageExtension()
    {
        if (function_exists('imagecreatefromjpeg'))
        {
            return true;
        }
        
        return false;
    }
    
    /**
    * @desc Gets the attributes of the image and creates an 
    *       image object in memory
    */
    public function prepareImage()
    {
        if (($this->_imageFile != '') && ($this->_image == null))
        {
            list ($this->_width, $this->_height, $this->_type, $attr) = 
                                        getimagesize ($this->_imageFile);
            
            switch ($this->_type)
            {
                case IWIMG_JPG:
                    $this->_image = imagecreatefromjpeg($this->_imageFile);
                    break;
                case IWIMG_GIF:
                    $this->_image = imagecreatefromgif($this->_imageFile);
                    break;
                case IWIMG_PNG:
                    $this->_image = imagecreatefrompng($this->_imageFile);
                    break;
                default:
                    break;
            }                    
        }		
    }
    
    /**
    * @desc Sets the image associated with this object.  Creates
    * an image object if file is found and image extension for PHP
    * exists.
    */
	public function setPath($filename)
	{    
		if (file_exists ($filename))
		{		
			$this->_imageFile = $filename;						
		}					
	}

    /**
    * @desc Retrieves the width of the source image
    * @return int The width of the source image.
    */
    public function getWidth()
    {
        if ($this->isValid())
        {
            return imagesx($this->_image);
        }
        else
        {
            return 0;
        }
    }

    /**
    * @desc Retrieves the height of the source image
    * @return int The height of the source image.
    */
    public function getHeight()
    {
        if ($this->isValid())
        {
            return imagesy($this->_image);
        }
    }
    
	/**
	 * Clears out all thumbnails for the given object type
	 *
	 * @param shItem $obj An object derived from shItem
	 * @return int Returns the number of thumbnails cleared out.
	 */
	public function clearThumbs($obj)
	{
		if ($obj instanceof shItemDocument)
		{		
			if ($obj->get(PROP_SUBTYPE_ID) == DOCTYPE_IMAGE)
			{
				if (is_dir (CACHE_DIR))		
				{
					$docname = iwUtilities::basenameNoExtension($obj->getInternalFilename());
					
					$files = scandir (CACHE_DIR);
					foreach ($files as $file)
					{
						if (strpos($file,$docname) !== false)
						{
							unlink(CACHE_DIR.'/'.$file);
						}
					}
				}
			}
		}		
	}
	
	public function destroy ()
	{
		if ($this->_image != null)
		{
			// Make sure and destroy the resource
			@imagedestroy ($this->_image);
			$this->_image = null;
		}		
	}

	public function getThumbnailAsString ($maxWidth,$maxHeight,$useCacheIfAvailable=true)
	{	
		$outpath = $this->getThumbnail('',$maxWidth,$maxHeight,$useCacheIfAvailable);
		if (file_exists ($outpath))
		{
			return base64_encode(file_get_contents ($outpath));
		}
		
		return '';
	}
		
    /**
    * @desc Generates a thumbnail of a given size and caches it.  If a thumbnail
    * of the given size already exists, then the cached thumbnail will be returned
    * instead.
    * @param string $destinationPath The location where the thumbnail should be generated. Optional.  If
    *                   none specified then the standard cache/thumbs dir will be used.
    * @param int $maxWidth The maximum width of the thumbnail.    
    * @param int $maxHeight The maximum height of the thumbnail
    * @param bool $useCacheIfAvailable If this is set to true then the cached thumb will be used
    *               If false then the cache will not be used even if it exists for this image.
    * @return string Returns the path to the thumbnail.
    */
	public function getThumbnail ($destinationPath='', $maxWidth=64, $maxHeight=64, $useCacheIfAvailable)
	{
        if (!function_exists('imagecreatetruecolor'))
        {
            app()->addError(sprintf(str('STR_INVALID_FUNCTION'), 'imagecreatetruecolor'));
            return '';
        }
        
		if ($useCacheIfAvailable)
		{                               
			$file = $this->findInCache($this->_imageFile,$maxWidth,$maxHeight);
			if ($file != '')
			{
				$inpath = CACHE_DIR.'/'.$file;
			
				$outpath = '';
				
				if ($destinationPath != '')
				{
					$outpath = $destinationPath.'/'.$file;
					copy ($inpath,$outpath);
				}
				else 
				{                         
					$outpath = CACHE_DIR_REL.'/'.$file;	
				}
				
				if (IW_CONFIGURABLE_RELPATH != '')
				{
					// This is only needed when the bridge is 
					//	being used and the active file is not in 
					//	Nexus root.
					
					if (defined('IWONDER_INTERNAL_SCRIPT') != true)
					{
						$outpath = IW_CONFIGURABLE_RELPATH.'/'.$outpath;
					}
				}                
				return $outpath;
			}
		}
        
        // Creates the image object in memory and populates the dimensions
        //  properties of this object.
        $this->prepareImage();
                
		$thumb = null;
		
		// First check to see if it needs to be shrunk.
		$newheight = $this->_height;
		$newwidth = $this->_width;
		
		if ($this->_width > $maxWidth)
		{
			$newwidth = $maxWidth;
			$newheight = ($newwidth/$this->_width)*$this->_height;
			
			// Make sure the height hasn't been calculated out of existance.
			if ($newheight < 1) $newheight = 1;
		}
		
		if ($newheight > $maxHeight)
		{
			$newheight = $maxHeight;
			$newwidth = ($newheight/$this->_height)*$this->_width;
		}			
		
		$thumb = @imagecreatetruecolor($newwidth, $newheight);        
		@imagecopyresampled($thumb, $this->_image, 0, 0, 0, 0, $newwidth, $newheight, $this->_width, $this->_height);
		
		if ($thumb)
		{
			$name = iwUtilities::basenameNoExtension($this->_imageFile);
			$ext = iwUtilities::getExtension($this->_imageFile);			
			$thumbfilename = $name.THUMB_FILENAME_SEPARATOR.$maxWidth.
								THUMB_FILENAME_SEPARATOR.$maxHeight.
								THUMB_FILENAME_SEPARATOR.date('m-d-Y').
								THUMB_FILENAME_SEPARATOR.date('h-i-s').
								'.'.$ext;
			
			$inpath = CACHE_DIR.'/'.$thumbfilename;
			
			if ($destinationPath != '')
			{
				$outpath = $destinationPath . $thumbfilename;								
			}
			else 
			{
				$outpath = CACHE_DIR_REL.'/'.$thumbfilename;
				
				// This is only needed when the bridge is 
				//	being used and the active file is not in 
				//	Nexus root.
				
				if (defined('IWONDER_INTERNAL_SCRIPT') != true)
				{
					$outpath = IW_CONFIGURABLE_RELPATH.'/'.$outpath;
				}					
			}
			
			switch ($this->_type)
			{
				case IWIMG_JPG:
					imagejpeg ($thumb, $inpath);
					break;
				case IWIMG_PNG:
					imagepng ($thumb, $inpath);
					break;
				case IWIMG_GIF:
					imagegif ($thumb, $inpath);
					break;
				default:
					app()->addError (sprintf(str('STR_IMAGE_UNKNOWNTYPE'),$this->_type), ERROR_TYPE_SERIOUS);
					break;
			}
				
			if ($outpath != '')		
			{
				@copy ($inpath, $outpath);	
			}
			
			imagedestroy ($thumb);		
			
			return $outpath;		
		}			
		
		return app()->get('template_dir').'/images/invalid_thumb.jpg';
	}
		
	protected function findInCache ($sourcePath, $maxWidth, $maxHeight)
	{
				
		$nameToFind = iwUtilities::basenameNoExtension ($sourcePath);
		if (is_dir (CACHE_DIR))		
		{
			$files = scandir (CACHE_DIR);
			foreach ($files as $file)
			{
				$ext = strtolower(iwUtilities::getExtension($file));
				if (in_array($ext,array('jpg','jpeg','gif','png')))
				{
					// This is an image file, now check the name.
					$name = iwUtilities::basenameNoExtension($sourcePath);
					$cacheName = iwUtilities::baseNameNoExtension($file);
					@list($origName,$width,$height,$date,$time) = split (THUMB_FILENAME_SEPARATOR,$cacheName);
					if ($name == $origName)
					{
						if (($height == $maxHeight) && ($width == $maxWidth))
						{
							return $file;
						}
					}
				}
			}
		}
		
		return '';
	}


}

?>
