<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * The object collection class provides a common interface to 
 *	sets of iwObject based lists of items.  By storing objects
 *	in a collection, users will have access to methods to display
 * 	those items in a common format, sorting, and more.
 */
abstract class iwObjectCollection extends iwObject
{
		
	/**
	 * The array of objects contained in the collection
	 */
	protected $_objects = array();
	
	/** 
	 *	This is the total number of objects that are in the query that is being used in the 
	 * load function.  The number of objects stored in _objects may be a subset of that total
	 * so use this number to determine the actual total.
	 */
	protected  $_total = 0;
	
	/**
	 * The subtitle used when displaying this collection of objects
	 */
	protected $_subtitle = '';
	
	/**
	 * The title used when displaying this collection of objects
	 */
	protected $_title = '';
	
	/**
	 * The path to the icon image to use in the header
	 */
	protected $_icon = '';
	
	/**
	 * The most recently used parameters for loading data into the collection
	 */
	protected $_params = null;

	/**
	 * If false, any filtering UI elements are hidden
	 */
	protected $_showFilter = true;
	
	/**
	 * If false, any sorting UI elements are hidden
	 */
	protected $_enableSorting = true;
	
	/**
	 * If false, no checkboxes will appear beside the items
	 */
	protected $_enableSelection = true;
	
	/**
	 * The default constructor creates an empty parameter object. 
	 */
	public function __construct ()
	{
		$this->_params = new iwParameterCollection;
	}
	
	/** 
	 * Loads objects into the collection based on the parameters given
	 * 	in the parameter collection.  Each collection type will expect 
	 *	different parameters.  See the documentation for this method
	 *	overridden in those derived classes for more information about htem.
	 * 	@param iwParameterCollection $params The collection of parameters used 
	 *				to generate the list of objects contained within.
	 *	@param bool $countOnly If this is true, then the objects are not loaded, 
	 *				but the number of objects that would be in the collection is
	 *				stored (use the 'count' method to get that number.)
	 *	@return bool Returns true if the collection was loaded, false otherwise.
	 */
 	abstract public function load (iwParameterCollection $params,$countOnly=false);
 	
	/**
	 * Gets the name of the class of objects contained within this collection
	 *	@return string The name of the class.  If empty then the collection can
	 *					contain more than one type of object.
	 */
	abstract public function getClassName();
	
	/** 
	 * Retrieves the total number of objects in the collections
	 *	 @return int The number of items in the collection
	 */
 	public function getTotal ()
 	{
 		return $this->_total;
 	}
 	
 	function setAllowSelection($selection)
 	{ 		
 		$this->_enableSelection = $selection;
 	}
 	
	function setShowFilter($show)
	{
		$this->_showFilter = $show;
	}

	function setEnableSorting($enable)
	{
		$this->_enableSorting = $enable;
	}

	/**
	 * Sets the path to the icon to use in this collection's header display
	 *	@param string $icon The path to the icon image or empty string for 
	 *				no icon (which is the default)
	 */
 	public function setIcon ($icon)
 	{
 		$this->_icon = $icon;
 	}
 	
	/**
	 * Retrieves the path to the collection's header icon
	 *	@return string The path to the icon.  Empty means no icon has been set.
	 */
 	public function getIcon ()
 	{
 		return $this->_icon;
 	}
 	
	/**
	 * Sets the title that will be used in the collection's header area when
	 *	being displayed.
	 *	@param string $title The title string
	 */
 	public function setTitle ($title)
 	{
 		$this->_title = $title;
 	}

	/**
	 * Sets the subtitle to use  in the collection's header area when
	 *	being displayed
	 *	@param string $subtitle The subtitle string.
	 */
 	public function setSubtitle ($subtitle)
 	{
 		$this->_subtitle = $subtitle;
 	}

	/**
	 * Retrieves the currently stored title for this collection
	 * 	@return string The title stored.
	 */
 	public function getTitle ()
 	{
 		return $this->_title;
 	}

	/**
	 * Retrieves the currently stored subtitle for this collection 
	 *	@return string The subtitle stored
	 */
 	public function getSubtitle ()
 	{
 		return $this->_subtitle;
 	}	 	
	
	/**
	 * Gets the stored objects as html.  For this generic 
	 * 	lister to work, the derived class must override all abstract 
	 *	methods.  Sorting and filtering is supported in the display
	 *	of these items.  The method will look at get/post parameters to
	 *	determine what sorting/filtering to use.
	 *	The following parameters are used
	 *	@arg @c URL_SORTBY The value of this parameter is the database field to sort by
	 *	@arg @c URL_SORTDIR The value of this parameter can be 'asc' or 'desc'
	 *	@arg @c URL_FILTERED The value is not used.  But when this parameter is present, 
	 *				certain post variables are looked at to see about how to filter
	 *				the item.  See the derived class for more informationa about 
	 *				what can be filtered and how to do it.
	 *	@param int $maxPerPage The maximum number of items to show on each page.
	 *	@param int $currentPage The page to show.
	 *	@param bool $showHeader True if the header area should be shown, false otherwise.
	 * 	@return string Returns all the items as html
	 */
	public function html ($maxPerPage=10,$currentPage=1,$showHeader=true)
	{
		$smarty = app()->getSmartyObject();
		
		$smarty->assign('collection_image', $this->getIcon());
		$smarty->assign('collection_title', $this->getTitle());
		$smarty->assign('collection_subtitle', $this->getSubtitle());
		$smarty->assign('collection_action', app()->_getURLWithParameters());
		$smarty->assign('collection_filters', $this->getFilterHtml());
				
		$currentSortBy = $this->_params->find_value(FILTER_SORTBY);				
		$currentSortDirection = $this->_params->find_value(FILTER_SORTDIR);		
		
		$sorting = '';
				
		if ($this->allowSorting())
		{		
			if ($this->count() > 0 )
			{
				foreach ($this->getSortableFields() as $displayName=>$fieldName)
				{
					$link = app()->_getURLWithParameters(array(URL_SORTBY,URL_SORTDIR)).'&'.URL_SORTBY.'='.$fieldName;
					$emphasis = false;
					if ($currentSortBy == $fieldName)
					{
						$link .= '&'.URL_SORTDIR.'='.($currentSortDirection=='desc'?'asc':'desc');
						$emphasis = true;
					}
					$sorting .= '&nbsp;&nbsp;&nbsp;<a '.($emphasis?'class="emphasis"':'').' href="'.$link.'">'.$displayName.'</a>';
				}
			}
		}
		
		if ($sorting != '')
		{
			$sorting = str('STR_SORT_BY').': '.$sorting;
		}
		
		$selection = '';
		
		if ($this->count() > 0 && $this->itemsAreSelectable())
		{
			$selection .= 'Selection: ';
			$selection .= '&nbsp;&nbsp;&nbsp;<span><a href="javascript:void(0);" onclick="checkAllItems(true, objectCollectionForm.'.$this->getClassName().')">Select All</a></span>';
			$selection .= '&nbsp;&nbsp;&nbsp;<span><a href="javascript:void(0);" onclick="checkAllItems(false, objectCollectionForm.'.$this->getClassName().')">Select None</a></span>';
		}
		
		$smarty->assign('collection_sorting', $sorting);
		$smarty->assign('collection_selections', $selection);
		
		$clrflt = app()->_getURLWithParameters(array('*filter'));		
		$smarty->assign('collection_clearfilterlink', $clrflt);
		$html = '';
		if ($showHeader)
		{
			$html = smarty_templ_fetch('collections/collection.header.tpl.html',$smarty);		
		}
			
		if ($this->count() == 0)
		{
			$html .= '<div class="nodata">'.str('STR_NO_ITEMS').'</div>';
			return $html;
		}
		
		$html .= '<form name="objectCollectionForm" id="objectCollectionForm" method="post" action="'.app()->_getURLWithParameters(false).'">';
		$user = app()->getLoggedInUserObject();
		
		$paginate = new iwPaginate;		
        $paginate->SetLinksFormat('&laquo; '.str('STR_BACK'),' &bull; ',str('STR_NEXT').' &raquo;');
		$paginate->setItemCount ($this->count());
		$paginate->SetItemsPerPage ($maxPerPage);
		$paginate->SetCurrentPage ($currentPage);
							
		$index = 0;
		$sortColumn = -1;
		$currentSortBy = @$_GET[URL_SORTBY];
		$currentSortDirection = @$_GET[URL_SORTDIR]==''?'asc':$_GET[URL_SORTDIR];
		
		$pageLinks = '<div class="pagelinks">'.$paginate->GetPageLinks ().'</div>';
				
		foreach ($this->_objects as $item)
		{						
			if ($this->queryChildItemForDisplay())
			{
				$html .= $item->renderAsListItem($this->itemsAreSelectable(), true);								
			}
			else
			{
				$html .= $this->renderAsListItem($item);
			}
		}
										
				
		// Only show the pager if there is more than one page.
		if ($paginate->getPageCount() > 1)
		{
			$html .= $pageLinks;
		}
		
		if ($this->itemsAreSelectable())
		{
			$formButtons = $this->getFormButtons ();				
			if ($formButtons != '')
			{
				$html .= '<div class="collectionButtons">'.$formButtons.'</div>';
			}
			
			$html .= '<input type="hidden" value="'.FORMTYPE_MULTI.'" name="'.FORM_TYPE.'"/>';
		}
		
		$html .= '</form>';		
		
		return $html;
	}
 	
	/** 
	 * Gets the html for all the buttons that will be displayed along with 
	 *	the list of items. 
	 *	@return string The html of the form buttons
	 */
 	protected function getFormButtons ()
 	{
 		return '';
 	}
 	
	/**
	 * Gets the reference to the array of objects in this collection
	 *	@return array An array of iwObject-derived objects
	 */
 	public function &items ()
 	{
 		return $this->_objects;
 	}
 	
	/**
	 * Use this when you already know which items you want to include
	 *	in the collection.  This is an alternative to using the 
	 *	load method. 
	 *	@param array The array of items.  No check is done to verify that
	 *			these items are the same items expected by the collection
	 *			derivation if 'getClassName' returns an empty string.
	 */
	public function setItems ($items)
	{
		foreach ($items as $item)
		{
			$this->addItem ($item);
		}
	}
	
	/**
	 * Adds an item to the current collection.  If the class of the 
	 *	given object does not match the class expected by this collection
	 *	then the object is not added.
	 *	@param mixed The object to add.  
	 *	@return bool Returns true if the object was added, false otherwise.
	 */
 	public function addItem ($item)
 	{
		$givenClass = strtolower (get_class($item));
		$thisClass = strtolower ($this->getClassName());
		
		if ((strcasecmp($givenClass,$thisClass)==0) || 
			(strcasecmp ($thisClass,get_class($this)) == 0))
		{
 			array_push($this->_objects,$item);
 			$this->_total++;
 			
			return true;
		}
		
		return false;
 	}
 	
	/**
	 * This is an alias for the getTotal method
	 *	@return int The number of items in the collections
	 */
 	public function count ()
 	{
 		return $this->getTotal();
 	}
	
	/**
	 * This is an alias for the @c items method.
	 *	@return array The array of objects in the collection
	 */ 
	public function getAsArray ()
	{
		return $this->_objects;
	}

	/**
	 * Gets the object at the given index 
	 * 	@param int $index The index of the object to retrieve.
	 *	@return mixed Returns an iwObject-based object or null if not found
	 */
	public function getAt ($index)
	{
		if (count($this->_objects) > $index)
		{
			return $this->_objects[$index];
		}
		
		return null;
	}
	
	/**
	 * An alternative to the 'load' method that gives the user 
	 *	the ability to use a freeform query.  The query must 
	 *	yields rows that objects of this type (as specified in 
	 *	in getClassName) can read in using the setDBKeyValueArray
	 *	virtual method.
	 *	@param string $query The query to use to load the results
	 *	@return bool Returns true if the query was successful, false
	 *				otherwise.
	 */
	public function loadFromQuery ($query)
	{
		
        app()->flushCacheIfInvalid($query);
		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($rows !== false)
		{
			// Now load the data into the object.
			$className = $this->getClassName();
			foreach ($rows as $row)
			{
				// Check the doctype filter			
				$obj = new $className;
				$obj->setDBKeyValueArray ($row);
				$this->addItem ($obj);
			}

			return true;
		}

		return false;
	}

	/** 
	 * This should be overridden if the method queryChildItemForDisplay
	 * 	returns false.  This puts the responsibility of formatting the 
	 *	item in the hands of the collection, not the item itself.  This 
	 * 	is useful when creating a collection that can contain more than
	 * 	one type of item.
	 * 	@param mixed The object to display
	 * 	@return string The html to display the item.
	 */
	protected function renderAsListItem ($item,$details,$actions=true)
	{							
		return ('Error: This type of item cannot be displayed');		
	}
	
	/** 
	 * When this returns true, it means that when displaying the collection,
	 * 	the objects themselves should be queried for their appearance.  If 
	 *	false, then the collection will determine what each item will look like.
	 *	@return bool See description.
	 */
	protected function queryChildItemForDisplay()
	{
		return true;
	}
	
	/**
	 * Return true to show sorting control when displaying the collection
	 *	@return bool See description
	 */	
	function allowSorting ()
	{
		return $this->_enableSorting;
	}
		
	/**
	 * Return true to show selection controls when displaying the collection
	 *	@return bool See description
	 */	
	protected function itemsAreSelectable () 
	{
		return $this->_enableSelection;
	}
	
	/**
	 * Return an array of all the fields that can be sorted.  The format of the
	 *	associative array is as follows:
	 *		[display name]=>[database field]
	 *	@return array See description
	 */		
	protected function getSortableFields ()
	{		
		return array ();
	}	
	
	/**
	 * Determines what filters should be active for this collection and returns
	 *	an iwParameterCollection object that represents the filtering rules. 
	 *	@return iwParameterCollection The iwParameterCollection containing the parameters
	 *			that define the rules for the filter.
	 */
	abstract public function processFilters ();
}

?>
