<?php

/**
 * Figures out as best it can what the absolute path to the root
 *    of this website is (both in terms of the local path and the
 *    full URL.
 *    @param $absolutePath On output, contains the full path to the root of the site
 *            /user/home/iwonder/public_html/
 *    @param $urlPath On output, contains the url to the root of the site.
 */
function getPaths($file, &$absolutePath, &$urlPath)
{
    // Find the website's root and the absolute local path to the root of
    //    the website.
    $baseDir = dirname($file).'/';

    // automatically define the base url
    $baseUrl = ( isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') ? 'https://' : 'http://';
    $baseUrl .= isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : getenv('HTTP_HOST');
    $baseUrl .= dirname($_SERVER['PHP_SELF']);
    
    $absolutePath = $baseDir;
    $urlPath = $baseUrl;
}

/**
* @desc There are certain objects that need to be available everywhere in 
*       the code.  Some, such as the application object, is stored in the 
*       session and so is available each time the same user visits the 
*       site.  Others are generated each time a new page is served. The objects
*       that are created here are:
*       iwApplication > gApplication (session)
*       xajax > xajax (global)
*/
function initializeGlobals ()
{    
    if (!isset($_SESSION['gApplication']))
    {        
        $_SESSION['gApplication'] = new iwApplication;
    }
 
    $GLOBALS['gApplication'] = &$_SESSION['gApplication'];    
    
    $requestURL = IW_BASE_URL.'/index.php';
    
    // Hookup xajax to the AJAX server object.
    $GLOBALS['xajax'] = new xajax($requestURL);
    $GLOBALS['ajaxServer'] = new shAjaxServer();
	$GLOBALS['storage'] = new shLocalStorage();
    $GLOBALS['xajax']->registerCallableObject($GLOBALS['ajaxServer']);
	$GLOBALS['uploadServer'] = new shDocumentUploadServer();
    
    $GLOBALS['settings'] = new shSettings;
	$GLOBALS['typemanager'] = new shFileTypeManager;
    
    date_default_timezone_set(settings()->timezone);
}

/**
 * Retrieves the upload server object which is used for storing documents that are already on
 * disk to the library, to the filebox or to the "purgatory".
 */
function &uploads()
{
	return $GLOBALS['uploadServer'];
}

/**
 * Returns the storage system object used to persist data to disk
 * @return shDocumentStorage An shDocumentStorage-derived object instance.
 */
function &storage()
{
	return $GLOBALS['storage'];
}

/**
 * Gets the database instance
 * @return ADOConnection One of the various ADODB driver objects (probably ADODB_mysql)
 */
function &db()
{
    return app()->getDatabase();
}

/**
* @desc Retrieves a reference to the global application object
* @return iwApplication The application obejct reference
*/
function &app ()
{
    return $GLOBALS['gApplication'];
}

/**
 * @desc Retrieves a references to the global file type manager
 * @return shFileTypeManager The filetype manager object
 */
function &typeman()
{
	return $GLOBALS['typemanager'];
}

/**
* @desc Retrieves an instance of the global ajax object
* @return xajax A reference to the xajax object
*/
function &ajax ()
{
    return $GLOBALS['xajax'];
}
                    
/**
* @desc Returns the correct translation for the given string code
* @return string The translated string.
*/
function str($stringCode)
{
	$str = app()->loadString($stringCode,'engine_translations');
	
	$argCount = func_num_args();
	if ($argCount > 1)
	{
		$args = func_get_args();
        array_shift($args);
		$str = vsprintf($str, $args);
	}
    
	return $str; 
}

/**
* @desc Returns the correct translation for the given string code
* @return string The translated string.
*/
function str_templates($stringCode,$args=array())
{
    $s = app()->loadString($stringCode,'template_translations');
    return vsprintf($s,$args);
}

/**
* @desc Retrieves the settings object that stores application
* settings
* @return shSettings Returns a reference to the settings objects
*/
function &settings ()
{
    return $GLOBALS['settings'];
}

/**
* @desc The global PHP error handler
*/
function handleError($errno, $errstr, $errfile, $errline)
{    
    $msg = "PHP Error: $errno: $errstr in '$errfile' at line $errline";
    if (IW_DEBUG) 
    {    
        if ($errno == E_STRICT)  
        {    
            if (!IW_DEBUG_STRICT)
            {
                // Unless we are not in debug mode, hide strict related errors.
                return;
            }
        }
        
        // Display on screen as well as sending email.
        //   NOTE: Ignore serialization errors.  These are
        //       caused by invalid data stored in the database, but
        //       it seems to only happen when there is associated
        //       media file data.  Can't handle it.
        if (strpos ($errstr, 'unserialize') !== false)
        {
               if ($errno == 8)
               {
                   return;
               }
        }

        // Check to see if we're supposed to display these errors.
        if (ini_get('error_reporting') == 0)
        {
            return;            
        }
        
        $msg .= '<br/>'.iwError::getTrace(true);
        app()->addError($msg);        
    }  
	else
	{
		if (IW_LOG_PHP_ERRORS)
		{
			l($msg);
		}
	}  
}

/**
 * Unsets global variables that are not important.
 *
 * Unsets all global variables that are not important, which
 * has the effect of overriding PHP's register_globals directive.
 * Also does a check thatsomeone isn't trying to create a 
 * global variable through the URL query.
 *
 * @author http://www.zend.com/manual/faq.misc.php
 * @return void
 */
function unregister_GLOBALS()
{
    if (!ini_get('register_globals'))
    {
        return;
    }

    if (isset($_REQUEST['GLOBALS']))
    {
        die('GLOBALS overwrite attempt detected');
    }

    // Variables that shouldn't be unset
    $noUnset = array('GLOBALS', '_GET', '_POST', '_COOKIE', '_REQUEST', '_SERVER', '_ENV', '_FILES', 'table_prefix');

    $input = array_merge($_GET, $_POST, $_COOKIE, $_SERVER, $_ENV, $_FILES, isset($_SESSION) && is_array($_SESSION) ? $_SESSION : array());
    foreach ($input as $k => $v)
    {
        if (!in_array($k, $noUnset) && isset($GLOBALS[$k]))
        {
            unset($GLOBALS[$k]);
        }
    }
}

/**
* @desc Called by ADODB when outputing an error or, if in debug mode, any query
*/
function outputQuery($msg,$newline)
{
    $output = $msg;
    if ($newline)
    {
        $msg .= "\n";
    }
    
    iwUtilities::logToFile($output,'dblog.html');
}


?>
