<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * A company (also called organizations) object represents a record in
 * the companies table in the database.  A company is made up of one or more
 * groups.  Companies, along with groups, are the primary means of organizing 
 * users of your library
 */  
class shCompany extends iwDbTable
{
	/**
	 * These are the groups that are members of the company.
	 */	 	
	protected	$_groups = array();
    
    /**
    * @desc Automatically load groups when new data is set using setDBKeyValueArray
    */
    protected   $_autoLoadGroups = true;
	
	/**
	 * The company's properties are:
	 * 	@arg id
	 * 	@arg name
	 * 	@arg description
	 * 	@arg status
	 * 	@arg logo
	 * 	@arg intro_page
	 */	 	 	 	 	 	 	 	
	protected function addProperties () 
	{
		parent::addProperties ();
		$this->_addProperty (PROP_ID,0);
		$this->_addProperty (PROP_NAME,'');
		$this->_addProperty (PROP_DESCRIPTION,'');
		$this->_addProperty (PROP_STATUS,0);
    	$this->_addProperty (PROP_LOGO,'');
    	$this->_addProperty (PROP_INTRO_PAGE,'');
	}

	/** 
	 *		
	 */
	public function getTypeName ($asPlural) 
	{		
		return $asPlural ? str('STR_COMPANY_NAME_PLURAL') : str('STR_COMPANY_NAME');
	}
	
	/** 
	 *	Returns @c OBJECT_TYPE_COMPANY	
	 */
	public function getType () 
	{
		return OBJECT_TYPE_COMPANY;
	}
	
	/**
	 * Removes the member groups from memory (not the database)
	 */	 	 	
	public function clearGroups ()
	{
		$this->_groups = array();
	}
	
	/**
	 * Determines if the given group is part of this company
	 * @param int $groupId The ID of the group to look for
	 * @return bool Returns true if the group was found, false otherwise.	 	 
	 */
	public function findGroup ($groupId)
	{
		foreach ($this->_groups as $group)
		{
			if ($group->get(PROP_ID) == $groupId)
			{
				return true;
			}
		}
		
		return false;
	}	
	
	/**
	 * Gets the number of groups that are members of this company.  This simply
	 * checks the in-memory group list and does NOT do a database lookup.
	 * 	 	 
	 * @return in The number of groups.
	 */	 	 	
	public function getGroupCount()
	{
		return count($this->_groups);
	}
	
	/**
	 * Gets all the groups as an array
	 * @return array An array of group objects
	 */	 	 	
	public function getGroups ()
	{
		return $this->_groups;
	}
	
	/**
	 * Adds the given group to the list of member groups.  If the given 
	 * parameter is an integer, it will be assumed that it is the id of the
	 * group to be added and a group object will be created that has that ID
	 * (but no database lookup is done).  If it is a group object, then it 
	 * is simply stored in memory in the groups array
	 *	@param mixed Either an int or an shGroup object (see description)
	 */	 	 	 	 	 	 	
	public function addGroup ($group)
	{
		if (is_numeric ($group))
		{
			$gp = new shGroup;
			$gp->set(PROP_ID,$group);
			$this->_groups[] = $gp;
		}
		else
		{
			$this->_groups[] = $group;
		}
	}
	
	/**
	 * The load method is overridden so that the associated groups can be
	 * loaded into memory once the company details have been loaded
	 *	@return bool Returns true if the load was successful, false otherwise.
	 */	 	  	 	
	public function load ($loadContainedGroups=true)
	{
        $this->_autoLoadGroups = $loadContainedGroups;
		if (parent::load())
		{            
			return true;
		}
		
		return false;
	}
	
	/**
	 * The override will take care of updating the company > groups mapping
	 * table once the company has been added
	 */	 	 	
	public function add ()
	{
		if (parent::add())
		{
			$this->_updateMappings ();
			return true;
		}

		return false;
	}
	
	/**
	 * The override will take care of updating the company > groups mapping
	 * table once the company has been added
	 */	 	 	
	public function update ()
	{		
		if (parent::update())
		{		
			$this->_updateMappings ();            
			return true;
		}
		
		return false;
	}
	
	/**
	 * Clears out all the references in the company > groups mapping table.
	 * @return bool Returns true if the mappings were deleted successfully, false
	 * 				otherwise
	 */ 					 	 	 	
	protected function _deleteMappings ()
	{				
		// Remove existing mappings first
		$res = app()->getDatabase()->query ("DELETE FROM sh_company_group_map WHERE company_id='".$this->get(PROP_ID)."'");
		if ( ($res))
		{
			return true;
		}
		else
		{
			$this->addErrorObject ($res,ERROR_TYPE_SERIOUS);	
		}
		
		return false;
	
	}
		
	/**
	 * First removes all the mappings in the company groups mapping table then
	 * re-adds them based on the group objects current stored in this object.	 
	 * @return bool Returns true if the mappings were updated successfully, 
	 * 				false otherwise
	 */ 					 	 	 	
	protected function _updateMappings ()
	{
				
		if ($this->_deleteMappings ())
		{		
			foreach ($this->_groups as $group)
			{	
				$group_id = $group->get(PROP_ID);
								
				// Add company and group to respective mapping tables.
				$fields_values = array('company_id'=>$this->get(PROP_ID),
									   'group_id'=>$group_id);
									   
				$ret = app()->getDatabase()->AutoExecute('sh_company_group_map', $fields_values, 'INSERT');
	            app()->invalidateTable('sh_company_group_map');
                
				if ($ret === false)
				{
					$this->addErrorObject ($ret,ERROR_TYPE_SERIOUS);
				}						
			}			
		}		
	}
	
    public function deleteItemReferences()
    {
        $query = 'select * from sh_item_access_map where company_id = '.$this->get(PROP_ID);
        $res = db()->Execute($query);
        if ($res)
        {
            foreach($res as $row)
            {
                $count = db()->GetOne('select count(*) from sh_item_access_map where item_id='.$row['item_id']);
                if ($count > 1)
                {
                    // if there's another company that has access then just delete the record
                    db()->Execute('delete from sh_item_access_map where company_id='.$this->get(PROP_ID).' and item_id='.$row['item_id']);            
                }
                else                                                                         
                {
                    // If this is the only company that has access then switch to default company which should always be there.
                    db()->Execute('update sh_item_access_map set company_id=1,group_id=1  where company_id='.$this->get(PROP_ID).' and item_id='.$row['item_id']);            
                }
            }
            
            return true;
        }
                
        return false;
    }
    
    public function deleteCategoryReferences()
    {
        $query = 'select * from sh_category_access_map where company_id = '.$this->get(PROP_ID);
        $res = db()->Execute($query);
        if ($res)
        {
            foreach($res as $row)
            {
                $count = db()->GetOne('select count(*) from sh_category_access_map where category_id='.$row['category_id']);
                if ($count > 1)
                {
                    // if there's another company that has access then just delete the record
                    db()->Execute('delete from sh_category_access_map where company_id='.$this->get(PROP_ID).' and category_id='.$row['category_id']);            
                }
                else                                                                         
                {
                    // If this is the only company that has access then switch to default company which should always be there.
                    db()->Execute('update sh_category_access_map set company_id=1,group_id=1 where company_id='.$this->get(PROP_ID).' and category_id='.$row['category_id']);            
                }
            }
            
            return true;
        }
                
        return false;
    }
    
    public function deleteUserReferences()
    {                
        // Always revert back to default company/group
        $res = db()->Execute('update sh_user_access_map set company_id=1,group_id=1  where company_id='.$this->get(PROP_ID).' and username="'.$row['username'].'"');            
        if (!$res)
        {
            return false;
        } 
        
        return true;
    }
        
	/**  
	 *	The following fields are returned in the array
	 * 	@arg company_id
	 * 	@arg name
	 * 	@arg description
	 * 	@arg status
	 * 	@arg logo
	 * 	@arg intro_page_html
	 */
	public function getDBKeyValueArray ($includeDBKey) 
	{
	
		$tableConstant = $this->getTableConstant();
		
		$fields = array
				(
					$GLOBALS[$tableConstant]['fields']['COMPANY_ID'] => $this->get (PROP_ID),
					$GLOBALS[$tableConstant]['fields']['COMPANY_NAME'] => $this->get (PROP_NAME),
					$GLOBALS[$tableConstant]['fields']['COMPANY_DESCRIPTION'] => $this->get (PROP_DESCRIPTION),
					$GLOBALS[$tableConstant]['fields']['COMPANY_STATUS'] => $this->get (PROP_STATUS),
					$GLOBALS[$tableConstant]['fields']['COMPANY_LOGO'] => $this->get (PROP_LOGO),
					$GLOBALS[$tableConstant]['fields']['COMPANY_INTRO_PAGE'] => $this->get (PROP_INTRO_PAGE),
		);
						
		return $fields;
	}
	
	/**  
	 *	@see getDBKeyValueArray for all the expected fields in the given array	
	 */
	public function setDBKeyValueArray ($dbFieldsAndValues) {
			
		$tableConstant = $this->getTableConstant();
		$this->set (PROP_ID, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['COMPANY_ID']]);
		$this->set (PROP_NAME, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['COMPANY_NAME']]);
		$this->set (PROP_DESCRIPTION, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['COMPANY_DESCRIPTION']]);
		$this->set (PROP_STATUS, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['COMPANY_STATUS']]);
		$this->set (PROP_LOGO, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['COMPANY_LOGO']]);
		$this->set (PROP_INTRO_PAGE, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['COMPANY_INTRO_PAGE']]);
		
        if ($this->_autoLoadGroups)
        {
		    $this->loadGroups();
        }
		
		return true;
	}
	
	protected function loadGroups()
	{
		$this->_groups = array ();
		$query = 'select company_id,group_id from sh_company_group_map as scgm ' .
				'WHERE scgm.company_id = '.$this->get(PROP_ID);
				
        app()->flushCacheIfInvalid($query);
		$results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($results !== false)
		{
			foreach ($results as $result)
			{
				$group = new shGroup;
				$group->set(PROP_ID, $result['group_id']);
				if ($group->load ())
				{
					$this->_groups[] = $group;
				}
			}
		}	
		else
		{
			iwUtilities::logDatabaseError();
		}	
	}
	
	/**  
	 *	If $isRestoring is true then this will return @c COMPANY_STATUS_ACTIVE as
	 *		the value of the array.  otherwise it will return 
	 *		@c COPMANY_STATUS_DELETED	 	 	
	 */
	protected function getDBKeyValueForDelete ($isRestoring=false) 
	{
			$fields = array ($GLOBALS[$this->getTableConstant()]['fields']['COMPANY_STATUS'] => $isRestoring?COMPANY_STATUS_ACTIVE:COMPANY_STATUS_DELETED);								
			return $fields;
	}
	
	/**  
	 *	Returns @c 'TABLE_COMPANIES'	
	 */
	public function getTableConstant () 
	{
		return 'TABLE_COMPANIES';
	}
	
	/**  
	 *	The following fields are expected to be non-empty:
	 *	@arg name			 	
	 */
	public function requiredFieldsValid ($checkKey = false) {
	
		$invalidField = false;
		
		if ($this->get(PROP_NAME) == '') {
			$this->addError (str('STR_COMPANY_INVALID_NAME'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}	
		
		return !$invalidField;
	}
	
	/**  
	 *	If fieldName is true then this will return the string @c 'company_id'
	 *	If fieldName is false then this will return an integer 
	 */
	public function getUniqueID ($fieldName=false) 
	{
		if ($fieldName) {
			return $GLOBALS[$this->getTableConstant()]['fields']['COMPANY_ID'];
		}
		else {
			return $this->get (PROP_ID);
		}
	}
	
	/**
	 */	
	public function render ($params=false)
	{
		$html = '';
		
		$groups = array();
		foreach ($this->_groups as $group)
		{
			$groups[]=
				array(
					'name'=>$group->get(PROP_NAME),
					'id'=>$group->get(PROP_ID),
					'description'=>$group->get(PROP_DESCRIPTION)
				);
		}
		
		$users = $this->getUsers();
		$userArray = array();
		foreach($users as $user)
		{
			$userArray[] = $user->toArray();
		}
		
		$item = array ('id'=>$this->get(PROP_ID), 
							'name'=>$this->get(PROP_NAME), 
							'description'=>$this->get(PROP_DESCRIPTION),
							'groups'=>$groups,
							'users'=>$userArray,
							'actions'=>$this->getAvailableActions());
		
		$item['logo_link'] = '';
		
		if ($this->get(PROP_LOGO))
		{
			$item['logo_link'] = 'download.php?type='.OBJECT_TYPE_COMPANY.'&id='.$this->get(PROP_ID);
		}
									
		$username = app()->getLoggedInUserObject()->get(PROP_USERNAME);
		app()->notify (EVENT_OBJECT_VIEWED,array('type'=>$this->getType(),'id'=>$this->getUniqueID(),'username'=>$username));
		
		$smarty = app()->getSmartyObject ();
		$smarty->assign ('company',$item);
		return smarty_templ_fetch('views/company.tpl.html',$smarty);												
	}
	
	/**
	 * Returns an array of shUser objects for all users who are members of this
	 * company/organization.
	 * @return array An array of shUser objects.
	 */
	public function getUsers()
	{
		$coll = new shUserCollection;
		$params = new iwParameterCollection;			
		$p = new iwParameterArray(FILTER_COMPANY);
		$p->addItem_nokey ($this->get(PROP_ID));				
		$params->add($p);			
		$coll->load($params);
		return $coll->items();		
	}
	
	/**
	 * Generates a link that will display the company's logo, if any.
	 * @return string Returns the link if a logo has been given or false otherwise.
	 */
	public function getLogoLink()
	{
		if ($this->logo)
		{
			return 'download.php?type='.OBJECT_TYPE_COMPANY.'&id='.$this->get(PROP_ID);
		}

		return '';
	}
	
	/**
	 * This will output directly to the server the data necessary to display
	 * or download the associated logo.	 
	 */	 	
	public function downloadLogo ()
	{
				
		// Get the content		
		$content = $this->get(PROP_LOGO);		
		if ($content)
		{			
			// Replace spaces with '_' for downloading.
			$name = str_replace(' ','_',$this->get(PROP_NAME));
			$name .= '.jpg';
			
			header('Content-Description: File Transfer');
			header('Content-Type: image/jpeg');
			header('Content-Length: ' . strlen($content));
			header('Content-Disposition: attachment; filename=' . $name);
			echo $content;
		}				
	}	
		
	/**
	 * Once a company is deleted, any groups or items that refer to that 
	 * company must be dealt with.  In the case of groups, that associated is
	 * simply removed.  If the group belongs to other companies, they aren't
	 * affected.  
	 */	 	 	 	 	
	protected function onAfterPermanentDelete ()
	{				
        $this->_deleteMappings();
        $this->deleteUserReferences();
        $this->deleteCategoryReferences();
        $this->deleteItemReferences();
	}

	/**
	 */
	public function getItemIcon ($large=true)
	{
				
		if ($large)
		{
			$icon = 'company_icon_large.jpg';
		}
		else
		{
			$icon = 'company_icon_small.jpg';
		}
		$icon = app()->getTemplateFile('images/'.$icon);
		return $icon;	
	}	

	/**
	 * The available actions for all users:
	 * 	@arg View
	 * The available actions for admin users:
	 * 	@arg Edit
	 * 	@arg Send To Trash
	 */	 	
	public function getAvailableActions ($data='')
	{
		$actions =  
			array (
				shPage::makeActionArray (str('STR_ACTION_VIEW'), 
								 app()->getTemplateFile('images/view_small.jpg'),
								'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWCOMPANY.'&'.URLARG_ID.'='.$this->get(PROP_ID),
								'', 
								str('STR_ACTION_VIEW'))								
			);
				

		$user = app()->getLoggedInUserObject();
		
		if ($user && 
			($user->getPermissions()->getPermissionGroup() == USER_TYPE_DOMAIN_ADMIN) &&
			($user->getCompany()->get(PROP_ID) == $this->get(PROP_ID)))
		{
			$actions[] = 
			shPage::makeActionArray (str('STR_ACTION_EDIT'), 
							 app()->getTemplateFile('images/edit_small.jpg'),
							'index.php?'.URLARG_ACTION.'='.URLACTION_MODCOMPANY.'&'.URLARG_ID.'='.$this->get(PROP_ID),
							'', 
							str('STR_ACTION_EDIT'));
		}
		
		if (app()->isAdminLoggedIn())
		{
						
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_TRASH'), 
							 app()->getTemplateFile('images/trash_small.jpg'),
							 sprintf ("javascript:areYouSure('delete', 'index.php?%s=%s&%s=%d')",URLARG_ACTION,URLACTION_DELCOMPANY,URLARG_ID, $this->get(PROP_ID)),
							 '', 
							 str('STR_ACTION_TRASH'));
							
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_VIEWHISTORY'), 
							 app()->getTemplateFile('images/history_small.jpg'),
							 'index.php?a=view_mod&filter_object_type='.OBJECT_TYPE_COMPANY.'&filter_object_id='.$this->get(PROP_ID),
							 '', 
							 str('STR_ACTION_VIEWHISTORY'));		
							
		}				
		
		return $actions;
	}

	/**
	 */		
	public function renderAsListItem ($selectable, $details, $actions=true)
	{
		$img = $this->getItemIcon(false);						
		
		$description = iwUtilities::trucateWithEllipsis (strip_tags ($this->get(PROP_DESCRIPTION)), 125);
		if ($description == '')
		{
			$description = str('STR_NO_DESCRIPTION');
		}
		
		$link = 'index.php?a='.URLACTION_VIEWCOMPANY.'&id='.$this->get(PROP_ID);
				
		$smarty =  app()->getSmartyObject();
		$smarty->assign ('collection_item_id', $this->get(PROP_ID));
		$smarty->assign ('collection_item_name', $this->get(PROP_NAME));
		$smarty->assign ('collection_item_show_details', $details);
		$smarty->assign ('collection_item_image', $img);
		$smarty->assign ('collection_item_selectable', $selectable);
        $smarty->assign ('collection_item_elementid', '');
		$smarty->assign ('collection_array_name', get_class ($this));
		$smarty->assign ('collection_item_specialinfo', $this->getGroupCount().' groups(s) in this company');
		$smarty->assign ('collection_item_description', $description);
		$smarty->assign ('collection_item_actions', $this->getAvailableActions());
		$smarty->assign ('collection_item_droptype', 'group');
		$smarty->assign ('collection_item_viewlink', $link);		
		$smarty->assign ('collection_item_draggable', true);
		$collection_item_html = smarty_templ_fetch('collections/collection.item.tpl.html',$smarty);
								
		return $collection_item_html;		
	}	
}

?>
