<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * Anything in the library is derived from this class.  Items are
 * capable of being categorized, tagged, tracked via history, etc.
 * @note Currently, there is only one type of item (Documents)
 */
abstract class shItem extends iwDbTable
{	
	/**
	 * An array of company/group pairs that have access
	 * to this item.
	 */	
	protected 	$_companyGroupPairs = array();
	
	/**
	 * An array of item IDs that have been associated with this
	 * item by the user
	 */
	protected 	$_relatedItemIds = array ();
	
	/**
	 * An array of attribute(tag) objects assigned to this item
	 */
	protected 	$_attributes = array ();
	
	/**
	 * An array of categories to which this item belongs
	 */
	protected 	$_categories = array ();
	
	/**
	 * Items have the following properties:
	 * 	@arg @c id 
	 *  @arg @c name
	 *  @arg @c description
	 * 	@arg @c status
	 * 	@arg @c type_id This is the type of item this is (for example, ITEM_TYPE_DOCUMENT)
	 *  @arg @c subtype_id This is a more detailed description of this item (for example, DOCTYPE_IMAGE applies to item with type_id of ITEM_TYPE_DOCUMENT)
	 *  @arg @c type_version This is the version of this type.  This is used when the data parameter can change over time.
	 *  @arg @c original_filename Used when the item is stored on disk rather than in the data parameter.
	 *  @arg @c category_id The category to which this item belongs.
	 *  @arg @c data Various data associated with this item.
	 * 	@arg @c creation_date The date this item was added to the database
	 *  @arg @c modified_date The date this item was last modified.
	 */
	protected function addProperties () 
	{
		parent::addProperties ();
		$this->_addProperty (PROP_ID,0);
		$this->_addProperty (PROP_NAME,'');
		$this->_addProperty (PROP_DESCRIPTION,'');
		$this->_addProperty (PROP_STATUS,'0');
		$this->_addProperty (PROP_TYPE_ID,'');
		$this->_addProperty (PROP_SUBTYPE_ID,'');		
		$this->_addProperty (PROP_TYPE_VERSION,'');
		$this->_addProperty (PROP_ORIGINAL_FILENAME,'');
		$this->_addProperty (PROP_DATA,'');
		$this->_addProperty (PROP_CREATION_DATE,'');
		$this->_addProperty (PROP_MODIFIED_DATE,'');		
	}

	/**
	 * Gets all the categories to which this item belongs
	 *
	 * @return array An array of all category object to which this item belongs
	 */
	public function getCategories ()
	{
		return $this->_categories;
	}
	
	/**
	 * Clears out the in-memory list of categories to which this
	 * item belongs
	 */
	public function clearCategories ()
	{
		$this->_categories = array();
	}

	/**
	 * Removes the given category from the internal array
	 * 	@param mixed $category Either the id or the shCategory object to remove
	 */
	public function removeCategory ($category)
	{
		$idToFind = 0;
		if (is_object ($category))
		{			
			$idToFind = $category->get(PROP_ID);
		}
		else
		{
			$idToFind = $category;
		}
		
		$categories = array();
		foreach ($this->_categories as $cat)
		{
			if ($cat->get(PROP_ID) != $idToFind)
			{
				// Remove from array
				$categories[] = $cat;				
				break;
			}
		}
		$this->_categories = $categories;
	}
	
	/**
	 * Removes the given category from the internal array
	 * 	@param mixed $attribute Either the id or the shCategory object to remove
	 */
	public function removeAttribute ($attribute)
	{
		$idToFind = 0;
		if (is_object ($attribute))
		{			
			$idToFind = $attribute->get(PROP_ID);
		}
		else
		{
			$idToFind = $attribute;
		}
		
		$attributes = array();
		foreach ($this->_attributes as $attrib)
		{
			if ($attrib->get(PROP_ID) != $idToFind)
			{
				// Keep in array
				$attributes[] = $attrib;				
			}
		}
		$this->_attributes= $attributes;
	}
		
	/**
	 * Adds a new category to the in-memory list of 
	 * 	categories to which this item belongs.  If the 
	 * 	given category is an ID then a new category object
	 * 	is created and an id assigned to it. Then the object
	 *  is added to the array.  User must call update or add
	 * 	to update the database with the information.	
	 * 	@param mixed $category An ID or shCategory object
	 */
	public function addCategory ($category)
	{
		if (is_object ($category))
		{			
			$this->_categories[] = $category;
		}
		else
		{
			$catOb = new shCategory();
			$catOb->set(PROP_ID, $category);
			$this->_categories[] = $catOb;
		}
	}

	/**
	 * Determines whether or not this item has an attribute
	 * with the given name.
	 * @param string $attribName The name of the attribute to look for
	 * @return Returns true if this item has the given attribute.
	 */
	public function hasAttribute ($attribName)
	{
		foreach ($this->_attributes as $attrib)
		{
			if ($attribName == $attrib->get(PROP_NAME))
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 *  Get the array of attribute objects
	 * 	@return array An array of shAttribute objects
	 */
	public function getAttributes ()
	{
		return $this->_attributes;
	}
	
	/**
	 * Remove all attribute objects from the in-memory list
	 */
	public function clearAttributes ()
	{
		$this->_attributes = array();
	}

	/**
	 * Adds an attribute that applies to this item
	 *	@param mixed Either an attribute ID or an attribute object
	 */
	public function addAttribute ($attribute)
	{
		if ($attribute instanceof shAttribute)
		{			
			$this->_attributes[] = $attribute;
		}
		else
		{
			$attributeOb = new shAttribute;
			$attributeOb->set(PROP_NAME, $attribute);
			$this->_attributes[] = $attributeOb;
		}
	}
	
	/**
	 * Return true if this document type supports attributes
	 *	@return bool Returns true if the attributes is supported
	 */
	public function allowAttributes () 
	{
		return true;
	}

	/**
	 * Gets the array of company/group pairs which have access to this
	 * 	item.
	 * @return array An array of shCompanyGroupPair objects
	 */
	public function getDomains ()
	{
		return $this->_companyGroupPairs;
	}

	public function doesCompanyHaveAccess($cid)
	{
		foreach($this->_companyGroupPairs as $pair)
		{
			if ($cid == $pair->getCompany()->get(PROP_ID))
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Determines if the domain has access to this document.  
	 * User can specify 0 for the group ID which will effetively
	 * ignore the group check and just check the company
	 *
	 * @param int $cid The ID of the company
	 * @param int $gid The ID of the group (0 to ignore)
	 * @return bool Returns true if access permitted, false otherwise.
	 */
	public function doesDomainHaveAccess($cid,$gid)
	{
		foreach($this->_companyGroupPairs as $pair)
		{
            // Is this company equal to the given company id?  If so, then
            //  the company is allowed.  There is a special case where the company
            //  can be anything (except COMPANY_UNREGISTERED) if the doc allows 
            //  "All Companies" access
            if ($pair->getCompany()->id == $cid || 
                (($cid > 0) && ($pair->getCompany()->id == COMPANY_ALL)))
			{
				if ($gid != 0)
				{
					if (in_array($pair->getGroup()->id, array($gid,GROUP_ALL)))
					{
						return true;
					}
				}
				else
				{
					// Ignore the group.
					return true;
				}
			}
		}
		
		return false;
	}
	
	/**
	 * Clears all objects from the list of copmany/group pairs
	 */
	public function clearCompanyGroupPairs ()
	{
		$this->_companyGroupPairs = array ();
	}
	
	/** 
	 * Adds a company/group pair object that has 
	 * access to this item. 
	 * @param mixed $company An id or shCompany object
	 * @param mixed $group An id or shGroup object
	 */ 		
	public function addCompanyGroup ($company,$group)
	{		
		$pair = new shCompanyGroupPair;
		if (is_object ($company))
		{			
			$pair->setCompany($company);
		}
		else
		{
			$comp = new shCompany;
			$comp->set(PROP_ID, $company);
			$pair->setCompany($comp);
		}

		if (is_object ($group))
		{			
			$pair->setGroup($group);
		}
		else
		{
			$gr = new shGroup;
			$gr->set(PROP_ID, $group);
			$pair->setGroup($gr);
		}
		
		$this->_companyGroupPairs[] = $pair;
	}

	/**
	 * Gets an array of item ids that are related to this item
	 * @return array An array of integers that are the IDs of the
	 *					related items
	 */
	public function getRelatedItems ()
	{
		return $this->_relatedItemIds;
	}
	
	/**
	 * Removes all related items from the in-memory list
	 */
	public function clearRelatedItems ()
	{
		$this->_relatedItemIds = array ();
	}
	
	/**
	 * Adds the given item ID to the list of related items
	 * @param int $item The id of the item.	
	 */
	public function addRelatedItem ($itemId)
	{
		foreach($this->_relatedItemIds as $id)
		{
			// check if it's already in the list
			if ($id == $itemId)
				return;
		}
		
		$this->_relatedItemIds[] = $itemId;
	}
	
	/**
	 * Loads the item from the database then fills in the 
	 * following associated data:
	 *	@arg Company/Group pairs that have access to this item
	 *  @arg Related Item IDs
	 * 	@arg Attributes
	 * 	@arg Categories 
	 */
	public function load ()
	{
		if (parent::load())
		{
			// Load the companies and groups;
			$this->loadDomains();
			$this->loadRelatedItems();
			$this->loadTags();
			
			$coll = $this->getParentCategories();			
			if ($coll)
			{
				$this->_categories = $coll->items();
			}

			return true;
		}
		return false;
	}
	
	/**
	 * Adds the item to the database, sets the creation and modification
	 * dates, then adds the associated data to the necessary tables.
	 *	@arg Company/Group pairs that have access to this item
	 *  @arg Related Item IDs
	 * 	@arg Attributes
	 * 	@arg Categories
	 */
	public function add ()
	{
		$this->set(PROP_CREATION_DATE,self::getDatabaseTimeString(time()));
		$this->set(PROP_MODIFIED_DATE,self::getDatabaseTimeString(time()));
		
		if (parent::add())
		{
			$this->_updateMappings ();
			return true;
		}

		return false;
	}
	
	/**
	 * Updates the item to the database, sets the modification
	 * date, then updates the associated data in the necessary tables.
	 *	@arg Company/Group pairs that have access to this item
	 *  @arg Related Item IDs
	 * 	@arg Attributes
	 * 	@arg Categories
	 */
	public function update ()
	{		
		$this->set(PROP_MODIFIED_DATE,self::getDatabaseTimeString(time()));					
		if (parent::update())
		{		
			$this->_updateMappings ();
			return true;
		}
		
		return false;
	}
    
    public function removeAllConversations()
    {
        $res = db()->Execute('delete from sh_item_conversations where item_id = '.$this->get(PROP_ID));
        if (!$res)
        {
            iwUtilities::logDatabaseError();
            return false;
        }
        return true;        
    }
    
    public function onAfterPermanentDelete()
    {
        // Remove all references to this item
        $this->_deleteMappings();     
        $this->removeAllConversations();       
    }
    
	/**
	 * Marks the item as delete and updates the modification date
	 */
	public function delete ()
	{
		$this->set(PROP_MODIFIED_DATE,self::getDatabaseTimeString(time()));
		return parent::delete();
	}
	
	/** 
	 */
	public function getTypeName ($asPlural) 
	{		
		return $asPlural ? 'items' : 'item';
	}
	
	/** 
	 */
	public function getType () 
	{
		return OBJECT_TYPE_ITEM;
	}

	/**  
	 *  The following fields are in the base item tables
	 *	@arg @c item_id (int)
	 * 	@arg @c item_name (varchar255)
	 * 	@arg @c item_description (varchar255)
	 * 	@arg @c item_type_id (varchar255)
	 * 	@arg @c item_subtype_id (varchar255)
	 *  @arg @c item_type_version (int)
	 *	@arg @c item_original_filename (varchar255)
	 *	@arg @c item_data (text)
	 * 	@arg @c item_created (datetime)
	 *	@arg @c item_modified (datetime)
	 * 	@arg @c item_status (tinyint)
	 */
	public function getDBKeyValueArray ($includeDBKey) 
	{
	
		$tableConstant = $this->getTableConstant();
				
		$fields = array
				(
					$GLOBALS[$tableConstant]['fields']['ITEM_ID'] => $this->get (PROP_ID),
					$GLOBALS[$tableConstant]['fields']['ITEM_NAME'] => $this->get (PROP_NAME),
					$GLOBALS[$tableConstant]['fields']['ITEM_DESCRIPTION'] => $this->get (PROP_DESCRIPTION),
					$GLOBALS[$tableConstant]['fields']['ITEM_TYPE_ID'] => $this->get (PROP_TYPE_ID),
					$GLOBALS[$tableConstant]['fields']['ITEM_TYPE_VERSION'] => $this->get (PROP_TYPE_VERSION),
					$GLOBALS[$tableConstant]['fields']['ITEM_ORIGINAL_FILENAME'] => $this->get (PROP_ORIGINAL_FILENAME),
					$GLOBALS[$tableConstant]['fields']['ITEM_DATA'] => $this->get (PROP_DATA),
					$GLOBALS[$tableConstant]['fields']['ITEM_CREATION_DATE'] => $this->get (PROP_CREATION_DATE),
					$GLOBALS[$tableConstant]['fields']['ITEM_MODIFIED_DATE'] => $this->get (PROP_MODIFIED_DATE),
					$GLOBALS[$tableConstant]['fields']['ITEM_STATUS'] => $this->get (PROP_STATUS),
					$GLOBALS[$tableConstant]['fields']['ITEM_SUBTYPE_ID'] => $this->get (PROP_SUBTYPE_ID)
		);
						
		return $fields;
	}
	
	/** 
	 * 	See getDBKeyValueArray for expected fields in given array 
	*/
	public function setDBKeyValueArray ($dbFieldsAndValues) {
			
		$tableConstant = $this->getTableConstant();
				
		$id = @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_ID']];
		if ($id != 0)
		{
			$this->set (PROP_ID, $id);
		}
		
		$this->set (PROP_NAME, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_NAME']]);
		$this->set (PROP_DESCRIPTION, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_DESCRIPTION']]);
		$this->set (PROP_STATUS, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_STATUS']]);
		$this->set (PROP_TYPE_ID,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_TYPE_ID']]);
		$this->set (PROP_SUBTYPE_ID,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_SUBTYPE_ID']]);
		$this->set (PROP_TYPE_VERSION,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_TYPE_VERSION']]);
	 	$this->set (PROP_ORIGINAL_FILENAME,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_ORIGINAL_FILENAME']]);	  
	 	$this->set (PROP_DATA,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_DATA']]);
		$this->set (PROP_CREATION_DATE,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_CREATION_DATE']]);			 
		$this->set (PROP_MODIFIED_DATE,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_MODIFIED_DATE']]);			
		$this->set (PROP_STATUS, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_STATUS']]);			  

		return true;
	}
	
	/**  
	 * If $isRestoring is true then this will return @c ITEM_STATUS_ACTIVE, otherwise it will return
	 * @c ITEM_STATUS_DELETED
	*/
	public function getDBKeyValueForDelete ($isRestoring=false) 
	{
			$fields = array ($GLOBALS[$this->getTableConstant()]['fields']['ITEM_STATUS'] => $isRestoring?ITEM_STATUS_ACTIVE:ITEM_STATUS_DELETED);								
			return $fields;
	}
	
	/**  
	 * Returns @c 'TABLE_ITEM'
	 */
	public function getTableConstant () 
	{
		return 'TABLE_ITEM';
	}
		
	/**  
	 * If fieldname is true then this will return @c 'item_id' otherwise it will return
	 * 	the unique ID of the item.
	*/
	public function getUniqueID ($fieldName=false) 
	{
		if ($fieldName) {
			return $GLOBALS[$this->getTableConstant()]['fields']['ITEM_ID'];
		}
		else {
			return $this->get (PROP_ID);
		}
	}
	
	/** 
	 * Required fields for items are:
	 * 	@arg name
	 *	@arg At least one company/group pair
	 *  @arg At least one category
	 */
	public function requiredFieldsValid ($checkKey = false) 
	{	
		$success = true;
		
		if ($this->get(PROP_NAME) == '') {
			$this->addError (str('STR_ITEM_NONAME'), ERROR_TYPE_SERIOUS);
			$success = false;
		}
		
		if (count($this->_companyGroupPairs) == 0)
		{
			$this->addError (str('STR_ITEM_NOGROUP'),ERROR_TYPE_SERIOUS);
			$success = false;
		}
		
		if (count($this->_categories) == 0)
		{
			$this->addError(str('STR_ITEM_NOCATS'));
			$success = false;
		}
				
		return $success;
	}
	
	/**
	 * Used to clean up references to this item in tables other than this one.
	 *	Cleans references in the following tables:
	 *		@arg sh_item_access_map
	 *		@arg sh_item_item_map
	 *		@arg sh_item_attribute_map
	 *		@arg sh_item_category_map
	 */
	protected function _deleteMappings ()
	{
		
		// Remove existing company/group mappings
		$res = db()->query ("DELETE FROM sh_item_access_map WHERE item_id='".$this->get(PROP_ID)."'");
		if (!$res)
		{
			iwUtilities::logDatabaseError();
			return false;
		}

		// Remove existing item mappings
		$query = "DELETE FROM sh_item_item_map WHERE source_item_id='".$this->get(PROP_ID)."'";
		$res = db()->query ($query);		
		if (!$res)
		{
			iwUtilities::logDatabaseError();
			return false;
		}

		// Remove existing attribute mappings
		$query = "DELETE FROM sh_item_attribute_map WHERE item_id='".$this->get(PROP_ID)."'";
		$res = db()->query ($query);		
		if (!$res)
		{
			iwUtilities::logDatabaseError();
			return false;
		}

		// Remove existing category mappings
		$query = "DELETE FROM sh_item_category_map WHERE item_id='".$this->get(PROP_ID)."'";
		$res = db()->query ($query);		
		if (!$res)
		{
			iwUtilities::logDatabaseError();
			return false;
		}
		
		return true;
	
	}
	
	/**
	 * Updates references in other tables.  The following tables are updated:
	 *	@arg @c sh_item_access_map 
	 *  @arg @c sh_item_item_map
	 * 	@arg @c sh_item_category_map
	 * 	@arg @c sh_item_attribute_map
	 */
	public function _updateMappings ()
	{
		if ($this->_deleteMappings ())
		{					
			foreach ($this->_companyGroupPairs as $pair)
			{	
				$group_id = $pair->getGroup()->get(PROP_ID);
				$company_id = $pair->getCompany()->get(PROP_ID);
								
				// Add company and group to respective mapping tables.
				$fields_values = array('item_id'=>$this->get(PROP_ID),
									   'group_id'=>$group_id,
									   'company_id'=>$company_id);
                                       								   
				$ret = app()->getDatabase()->AutoExecute('sh_item_access_map', $fields_values, 'INSERT');                
	
				if (!$ret)
				{
					iwUtilities::logDatabaseError();
				}						
			}		
            app()->invalidateTable('sh_item_access_map');
			
			foreach ($this->_relatedItemIds as $itemid)	
			{
				$relid = $itemid;
				$srcid = $this->get(PROP_ID);
				$fields_values = array ('source_item_id'=>$srcid, 'related_item_id'=>$relid);
				$ret = app()->getDatabase()->autoExecute('sh_item_item_map', $fields_values, 'INSERT');	                
				if (!$ret)
				{
					iwUtilities::logDatabaseError();
				}						
			}
            
            app()->invalidateTable('sh_item_item_map');

			foreach ($this->_attributes as $attrib)	
			{
				$attribid = $attrib->id;
				$itemid = $this->id;
				$fields_values = array ('item_id'=>$itemid, 'attribute_id'=>$attribid);		
				$ret = app()->getDatabase()->autoExecute('sh_item_attribute_map', $fields_values, 'INSERT');	
				if (!$ret)
				{
					iwUtilities::logDatabaseError();
				}						
			}
           app()->invalidateTable('sh_item_attribute_map');
            

			foreach ($this->_categories as $cat)	
			{
				$catid = $cat->get(PROP_ID);
				$itemid = $this->get(PROP_ID);
				$fields_values = array ('item_id'=>$itemid, 'category_id'=>$catid);
				$ret = app()->getDatabase()->autoExecute('sh_item_category_map', $fields_values, 'INSERT');	
				if (!$ret)
				{
					iwUtilities::logDatabaseError();
				}						
			}
            app()->invalidateTable('sh_item_category_map'); 
		}		
	}	

	/**
	 * Gets the collection object containing all the categories that are marked as being
	 *	parent categories.
	 *  @return shCategoryCollection The collection object containing all the categories
	 *				that are parent to this document.
	 */
	public function getParentCategories()
	{
        // This will only work with categories that are *not* root.  The root has an ID of 0
        //  so this will not return that as a category.
		$query = 'select sh_categories.* from sh_item_category_map, sh_categories where '.
					'item_id='.$this->get(PROP_ID).' AND sh_categories.category_id=sh_item_category_map.category_id';
					                    
		$coll = new shCategoryCollection;
		$coll->loadFromQuery ($query);

        // Now see if the root is in the list
        $query = 'select count(category_id) from sh_item_category_map where item_id='.$this->get(PROP_ID).' and category_id=0';
        $hasRootCategory = db()->CacheGetOne(DB_CACHE_EXPIRE,$query);
        if ($hasRootCategory)
        {
            $root = new shCategory;
            $root->set(PROP_ID, 0);
            $root->load();
            
            $coll->addItem($root);
        }
        
		return $coll;
	}

	/**
	 * Loads the domains into the internal array of shCompanyGroupPair objects
	 * Note that the full details of each company and group are note loaded.  Instead
	 * each company and group object holds only the ID.
	 */
	protected function loadDomains()
	{       
		$this->_companyGroupPairs = array ();
		$query = 'select company_id,group_id from sh_item_access_map as sam ' .
				'WHERE sam.item_id = '.$this->get(PROP_ID);
			
        app()->flushCacheIfInvalid($query);	
		$results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($results !== false)
		{
			$documents = array ();
			foreach ($results as $result)
			{                
				$pair = new shCompanyGroupPair;
				$group = new shGroup;                
				$group->set(PROP_ID, $result['group_id']);                
				$pair->setGroup ($group);

				$company = new shCompany;
				$company->set(PROP_ID, $result['company_id']);
				$pair->setCompany ($company);
				$this->_companyGroupPairs[] = $pair;
			}
		}	
		else
		{
			iwUtilities::logDatabaseError();
		}			
	}
	
	/**
	 * Loads all related items into the internal array irregardless of whether or
	 * not the current user has access to those items.  It's the responsibility of 
	 * the client to filter out those items which are unauthorized.
	 */
	protected function loadRelatedItems()
	{
		$this->_relatedItemIds = array();
		
		$array = array ();
		$query = 'select * from sh_item_item_map as sim ' .
				'WHERE sim.source_item_id = '.$this->get(PROP_ID);
				
        app()->flushCacheIfInvalid($query);
		$results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($results !== false)
		{
			$documents = array ();
			foreach ($results as $result)
			{
				$this->_relatedItemIds[] = $result['related_item_id'];				
			}
		}
	}
	
	/**
	 * Populates the internal list of tags (attributes)
	 */
	protected function loadTags()
	{
		$this->_attributes = array ();
		
		$array = array ();
		$query = 'select sim.*,sh_attributes.* from sh_item_attribute_map as sim,sh_attributes ' .
				'WHERE sim.item_id = '.$this->get(PROP_ID).' AND sh_attributes.attribute_id = sim.attribute_id';
				
        app()->flushCacheIfInvalid($query);
        
		$results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
        
		if ($results)
		{
			$documents = array ();
			foreach ($results as $result)
			{
				$attrib = new shAttribute();
				$attrib->setDBKeyValueArray($result);
				$this->_attributes[] = $attrib;						 
			}
		}
	}
	
	/**
	 * Indicates whether or not items of this type can be dragged in the interface
	 *	@return bool Returns true if items can be dragged.  False otherwise.
	 */
	public function isDraggable ()
	{
		return true;
	}	
			
	/**
	 * Outputs the appropriate html to force the browser to download
	 * this item as a file.
	 * @note THIS OUTPUTS DIRECTLY TO THE SERVER
     * @param bool $logDownload If true, then the download attempt will be logged otherwise
     *          no log will be added causing the download count to
     *          remain as is.
     */
    abstract public function downloadItem ($logDownload);
}
?>
