<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

define ('IMAGE_HEIGHT', 'iheight');
define ('IMAGE_WIDTH', 'iwidth');
define ('IMAGE_BOTH', 'iboth');

class document
{
	var $type;
	var $contents;
}

/**
 * The document type encompasses any type of data that is stored
 * as a file on disk.  Data in the library is stored on disk though
 * there is some information stored in the data field with certain
 * file types (such as images)
 */
class shItemDocument extends shItem
{
	/**
	 * This is the temporary location of files after they are uploaded.
	 */
	public $_temporaryPath = '';
    
	/**
	 * Adds no new properties to the items but sets the following:
	 *	@arg item_type_id is set to @c ITEMTYPE_DOCUMENT
	 *	@arg item_type_version is set to @c CURRENT_DOCUMENT_VERSION
	 *
	 */	
	protected function addProperties () 
	{
		parent::addProperties ();				
		$this->set(PROP_TYPE_ID, 'ITEMTYPE_DOCUMENT');
		$this->set(PROP_TYPE_VERSION, CURRENT_DOCUMENT_VERSION);		
	}
	
	/**
	 * This override will send emails to those who have registered for notification
	 * 	when documents have been added to the  given category 
	 */
	public function add () 
	{
		if (parent::add())
		{
			// Send notifications if necessary
			$notify = new shUserNotification;
			$notify->notifyAdd($this);
			
			return true;
		}
		
		return false;
	}

	/**
	 * This override will send emails to those who have registered for notification
	 * 	when documents in the given category have changed 
	 */
	public function update ()
	{
		if (parent::update())
		{
			// Send notifications if necessary
			$notify = new shUserNotification;
			$notify->notifyUpdate($this);
			return true;		
		}
		
		return false;
	}
	
	/**
	 * This will return the appropriate file type object 
	 * based on what the subtype stored with this item record
	 * @return shFileType A file type object or NULL if no match was found.
	 */
	public function getFileTypeObject()
	{        
		switch ($this->get(PROP_SUBTYPE_ID))
		{
			case DOCTYPE_IMAGE:
				return new shFileTypeImage;
			case DOCTYPE_HTML:
				return new shFileTypeHtml;
			case DOCTYPE_PDF:
				return new shFileTypePdf;
			case DOCTYPE_WORD:
				return new shFileTypeWord;
			case DOCTYPE_EXCEL:
				return new shFileTypeExcel;
			case DOCTYPE_ARCHIVE:
				return new shFileTypeArchive;
			case DOCTYPE_RTF:
				return new shFileTypeRtf;
            case DOCTYPE_TEXT:
                return new shFileTypeText;
			default:
				return new shFileTypeBinary;	
		}
	}
	
	/**
	 * When this function has a valid document associated with it
	 *	then the icon that matches the file type will be returned.
	 *	If no item matches the file type then the 'binary' icon
	 * 	will be returned.
	 *	If there is no valid document associated with this item then
	 *	the generic 'document' icon will be returned. 
	 */
	public function getItemIcon ($large=true)
	{		
		$fileType = $this->getFileTypeObject();		
		if ($fileType instanceof shFileType)
		{
			$icon = $fileType->getThumbnailPath($this, $large?48:16, $large?48:16, true);		
			return $icon;
		}
		
		return '';
	}
    
	/**
	 * The following actions can be performed when logged in as a regular user:
	 * 	@arg Add To Bin
	 *	@arg View
	 *	@arg Download
	 *	When an administrator is logged in, they can:
	 *	@arg Edit
	 *	@arg Send To Trash
	 */
	public function getAvailableActions ($data='')
	{
		$fromCatId = '';
		if (is_array($data) && isset ($data['fromcat']))
		{
			$fromCatId = $data['fromcat'];			
		}
		
		$actions = array();
		
		if (!shItemBin::isItemInBin ($this->get(PROP_ID)) && app()->isUserLoggedIn())
		{		
			$actions[] = 
				shPage::makeActionArray ('Add to Briefcase', 
							 app()->getTemplateFile('images/docbin_add_small.jpg'),
							'javascript:void(0);',
							'javascript:xajax_shAjaxServer.addSingleDocToBin('.$this->get(PROP_ID).')', 
							'Add to Briefcase',
							ACTION_BRIEFCASE);
		}		
		
		$viewLink = '';
		if ($fromCatId != '')
		{
			$viewLink = 
				'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWDOC.'&'.URLARG_ID.'='.$this->get(PROP_ID).'&'.URL_FROMCAT.'='.$fromCatId;
		}
		else
		{
			$viewLink =
				'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWDOC.'&'.URLARG_ID.'='.$this->get(PROP_ID);
		}
		
		$actions[] =  
				shPage::makeActionArray (str('STR_ACTION_VIEW'), 
								 app()->getTemplateFile('images/view_small.jpg'),
								$viewLink,
								'', 
								str('STR_ACTION_VIEW'),
                                ACTION_VIEW);
								
		$actions[] = 
				shPage::makeActionArray (str('STR_ACTION_DOWNLOAD'), 
								app()->getTemplateFile('images/download_small.jpg'),
								'download.php?'.URLARG_ID.'='.$this->get(PROP_ID),
								'', 
								str('STR_ACTION_DOWNLOAD'),
                                ACTION_DOWNLOAD);
		
		$user = app()->getLoggedInUserObject();
		if ($user && shAuthorize::canEditDocument($user->get(PROP_USERNAME),$this->get(PROP_ID)))
		{
			$actions[] = 
			shPage::makeActionArray (str('STR_ACTION_RENAME'), 
							 app()->getTemplateFile('images/rename_small.jpg'),
							'javascript:void(0);',
							'javascript:renameObject(\''.TYPE_PREFIX_DOC.'\',\''.$this->get(PROP_ID).'\')', 
							str('STR_ACTION_RENAME'),
                            ACTION_RENAME);
							
			$actions[] = 
			shPage::makeActionArray (str('STR_ACTION_EDIT'), 
							 app()->getTemplateFile('images/edit_small.jpg'),
							'index.php?'.URLARG_ACTION.'='.URLACTION_MODDOC.'&'.URLARG_ID.'='.$this->get(PROP_ID),
							'', 
							str('STR_ACTION_EDIT'),
                            ACTION_EDIT);
			
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_TRASH'), 
							 app()->getTemplateFile('images/trash_small.jpg'),
							 'index.php?'.URLARG_ACTION.'='.URLACTION_DELDOC.'&'.URLARG_ID.'='.$this->get(PROP_ID),
							 '', 
							 str('STR_ACTION_TRASH'),
                             ACTION_TRASH);
							
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_VIEWHISTORY'), 
							 app()->getTemplateFile('images/history_small.jpg'),
							 'index.php?a=view_mod&filter_object_type='.OBJECT_TYPE_ITEM.'&filter_object_id='.$this->get(PROP_ID),
							 '', 
							 str('STR_ACTION_VIEWHISTORY'),
                             ACTION_VIEW);		
			
		}				
		
        nx_fire_event('EVENT_DOCUMENT_ACTIONS_NEEDED',array('actions'=>&$actions,'doc'=>&$this));
        
		return $actions;
	}
	
	/**
	 * This gets the name of the item type as it appears in urls after 
	 *	the action prefix: view_[text] or mod_[text]
	 *	@return string The action text for this item type
	 */
	public function getUrlActionText()
	{
		return "doc";
	}
	
	/**
	 * Renders the document as a single line of text/images.  This is used
	 * in areas OTHER THAN the main browser view.
	 */
	public function renderAsListItem ($selectable, $details, $actions=true)
	{
		$img = $this->getItemIcon(false);						
		
		$description = iwUtilities::trucateWithEllipsis (strip_tags ($this->get(PROP_DESCRIPTION)), 125);
		$link = 'index.php?a=view_'.$this->getUrlActionText().'&id='.$this->get(PROP_ID);
			
		$smarty =  app()->getSmartyObject();
		$smarty->assign ('collection_item_id', $this->get(PROP_ID));
		$smarty->assign ('collection_item_name', $this->get(PROP_NAME));
		$smarty->assign ('collection_item_show_details', $details);
		$smarty->assign ('collection_item_image', $img);
		$smarty->assign ('collection_item_elementid', TYPE_PREFIX_DOC.'_'.$this->get(PROP_ID).'_name');
		$smarty->assign ('collection_item_selectable', $selectable);
		$smarty->assign ('collection_array_name', get_class($this));
		$smarty->assign ('collection_item_specialinfo', $this->get(PROP_ORIGINAL_FILENAME));
		$smarty->assign ('collection_item_description', $description);
        
        if ($actions)
        {
		    $smarty->assign ('collection_item_actions', $this->getAvailableActions());
        }
        
		$smarty->assign ('collection_item_droptype', $this->getUrlActionText());
		$smarty->assign ('collection_item_viewlink', $link);		
		$smarty->assign ('collection_item_draggable', $this->isDraggable());			
		$collection_item_html = smarty_templ_fetch('collections/collection.item.tpl.html',$smarty);
								
		return $collection_item_html;		
	}	

	/**
	 * Override this if the dervied document is not in the library (FileBox documents, 
	 *	for example, are not in the library).
	 * 	@return bool True if in the library, false otherwise.
	 */
	public function inLibrary()
	{
		return true;
	}
	
    public function toArray()
    {
        return $this->_properties;
    }
    
	/**
	 * The following smarty variables are prepared and returned in an associative array
	 *	@arg id
	 *	@arg name
	 *	@arg description
	 *	@arg original_filename
	 *	@arg modified_date
	 * 	@arg categories (name, id)
	 * 	@arg taglist
	 *	@arg content
	 * 	@arg icon
	 */
	public function getDetailedDocumentInfo()
	{
		
        $username = app()->getLoggedInUsername();
        
		// GENERAL INFO
		$array = array ('id'=>$this->get(PROP_ID), 
							'name'=>iwUtilities::unescapeString($this->get(PROP_NAME)), 
							'description'=>iwUtilities::unescapeString(iwUtilities::makeStringJavascriptSafe($this->get(PROP_DESCRIPTION))),
                            'status'=>$this->get(PROP_STATUS),
							PROP_ORIGINAL_FILENAME=>$this->get(PROP_ORIGINAL_FILENAME),
							PROP_MODIFIED_DATE=>iwUtilities::dateFormat($this->get(PROP_MODIFIED_DATE),false));
						
		// STATS
		$query = 'select count(*) as total from iw_modifications where subject_id='.$this->get(PROP_ID).' AND subject_type='.$this->getType().
					' AND (action="'.EVENT_OBJECT_DOWNLOADED.'")';
                    
        app()->flushCacheIfInvalid($query);
		$count = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);

		if ($count !== false)	
		{																	
			$array['stats'] = array ('downloads'=>$count[0]['total']);
		}

		// RELATED ITEMS
		if ($this->inLibrary())
		{		
			$relatedIds = $this->getRelatedItems();
			$relatedList = array();
			foreach ($relatedIds as $related)
			{
				if (shAuthorize::canViewDocument($username, $related))
				{
					$doc = new shItemDocument();
					$doc->set(PROP_ID, $related);
					if($doc->load())
					{
						$relatedList[] = array('name'=>$doc->get(PROP_NAME), 'image'=>$doc->getItemIcon(false), 'id'=>$doc->get(PROP_ID));									
					}
				}
			}
			$array['related'] = $relatedList;
		}
							
		// CATEGORIES
		$array['in_library'] = $this->inLibrary();
		if ($this->inLibrary())
		{		
			$parentCategories = $this->getParentCategories();
			$cats = array();
			if ($parentCategories)
			{
				foreach ($parentCategories->items() as $cat)
				{
                    if (shAuthorize::canViewCategory($username,$cat->get(PROP_ID)))
                    {
                        $cats[] = array('name'=>$cat->get(PROP_NAME), 'id'=>$cat->get(PROP_ID));				
                    }
				}
			}
			$array['categories'] = $cats;
		}
				
		// TAGS
		$array['taglist'] = '';
		if ($this->allowAttributes())
		{		
			$tags = new shAttributeCollection;
			$tags->setItems ($this->_attributes);
			$array['taglist'] = $tags->getTagsAsHtml($this->get(PROP_ID));
		}
		
        $conversationPosts = $this->getConversation();
        $array['conversations'] = array();
        foreach($conversationPosts as $post)
        {
            $p = $post->toArray();
            $p['datePosted'] = iwUtilities::dateTimeFormat($p['datePosted'],true);
            $array['conversations'][] = $p;
        }
				
        // DOMAINS
        $array['domains'] = array();
        if (app()->isAdminLoggedIn())
        {            
            $domains = $this->_companyGroupPairs;
            foreach($domains as $domain)
            {                   
				$domain->loadDetailedInfo();
				
                $cid = $domain->getCompany()->get(PROP_ID);
                $cname = $cid > 0 ? $domain->getCompany()->get(PROP_NAME) : ($cid < 0 ? str('STR_EVERYONE') : str('STR_COMPANY_ALL'));
                $gid = $domain->getGroup()->get(PROP_ID);
                $gname = $gid > 0 ? $domain->getGroup()->get(PROP_NAME) : ($cid < 0 ? str('STR_EVERYONE') : str('STR_GROUP_ALL'));
                
                $array['domains'][] = array('companyid'=>$cid,
                                          'companyname'=>$cname,
                                          'groupid'=>$gid,                                     
                                          'groupname'=>$gname);
            }
        }
        
		// Get the content											
		$pathToFile = storage()->retrieveDocument($this);
		if (file_exists($pathToFile))
		{
            $content = false;
            nx_fire_event('EVENT_DOCUMENT_PREVIEW_NEEDED',array('doc'=>&$this,'content'=>&$content));
                        
            if (!$content)
            {             
                $fileType = $this->getFileTypeObject();                
                if ($fileType)
                {
                    $content = $fileType->getPreview($this, $pathToFile);
                }
            }
					
			// Display the link to the document so that users can easily download it.
			$size = iwUtilities::byteFormat(storage()->getFileSize($this));
			
			$array['icon'] = $this->getItemIcon(true);
			$array['download'] = "download.php?t={$this->getType()}&id={$this->get(PROP_ID)}";
			$array['content'] = $content;
			
			$array['stats']['size'] = $size;		
			$array['stats']['last_modified'] = iwUtilities::dateTimeFormat($this->get(PROP_MODIFIED_DATE), true);
			$array['stats']['created_on'] = iwUtilities::dateTimeFormat($this->get(PROP_CREATION_DATE), true);
			$array['stats']['original_filename'] = $this->get(PROP_ORIGINAL_FILENAME);
		}
		else
		{
			$array['icon'] = $this->getItemIcon(true);
			$array['download'] = "";
			$array['content'] = '[File Not Found On Disk]';
			
			$array['stats']['size'] = '[No File]';		
			$array['stats']['last_modified'] = '[No File]';
			$array['stats']['created_on'] = '[No File]';
			$array['stats']['original_filename'] = '[No File]';			
		}
		
        storage()->removeTempDocument($this,$pathToFile);
        
		return $array;
	}
	
    public function getInternalFileName()
    {
        $filename = $this->get(PROP_ORIGINAL_FILENAME);
        
        if ($filename != '')
        {    
            $parts = pathinfo($filename);
            
            $ext = isset($parts['extension']) ? '.'.$parts['extension']:'';
            return '__document_'.sprintf ("%04d", $this->get(PROP_ID)).$ext;
        }

        return '';
    }
    
	/**
	 * This overrides the get and does special processing on the following items
	 *	@arg PROP_DOC_TYPE - Returns the subtype id of the document.
	 */
	public function get ($key)
	{
		if ($key == PROP_DOC_TYPE)
		{
			return $this->getDocumentType();
		}
		
		return parent::get ($key);
	}
	
	/**
	 * An alias for retrieving the subtype ID of the document.  Helps make 
	 * code that uses this class a little easier to read since that is what
	 *	is being stored in the subtype field.
	 * 	@return string The subtype.
	 */
	public function getDocumentType ()
	{
		return $this->get(PROP_SUBTYPE_ID);
	}
				
	/**
	 * This method is used internally to make the process of saving
	 * 	text entered by the user exactly the same as the process of 
	 *	uploading a document from the user's computer.  This will 
	 *	simply save the file to disk using the given name as the 
	 *	filename (spaces replaced with underscores (_)).   Note that 
	 *	this will also set the item subtype to @c DOCTYPE_HTML
	 * 	@param string $text The text to save.
	 */
	public function loadDocumentFromText ($text)
	{
		$name = $this->get(PROP_NAME);
		$name = str_replace (' ','_',$name);
		$name .= '.html';
		
		$dir = app()->get('upload_dir');
		$dir .= '/'.$name;
		
		// Write the file to temporary location
		$fp = fopen ($dir,'w+');
		if ($fp)
		{
			fwrite ($fp, $text);
			fclose ($fp);
			$this->_temporaryPath = $dir;
		}
		else
		{
			$this->addError (sprintf(str('STR_DOC_FILENOTCREATED'),$dir), ERROR_TYPE_SERIOUS);
		}
		
		$this->set(PROP_ORIGINAL_FILENAME, $name);
		$this->set(PROP_SUBTYPE_ID, DOCTYPE_HTML);
	}
	
	/**
	 * This will analyze the given file stored in a temporary location
	 * 	and save information such as the original filename, the document
	 * 	type and any other data that this particular type of file 
	 *	requires to be saved (such as image size for image types).
	 *	@param string $docType The type of document this is (e.g. @c DOCTYPE_IMAGE)
	 * 	@param string $name The name of the file (no path, just filename with extension)
	 *	@param string $path The full path to the file on disk.  This should
	 *					be the temporary location where the file was uploaded.
	 *  @param bool Returns true if the given document could be found, otherwise returns false.
	 */
	public function loadDocument ($docType, $name, $path)
	{	
		if (file_exists ($path))
		{			
			$this->_temporaryPath = $path;
			
			$this->set(PROP_ORIGINAL_FILENAME, $name);
			$this->set(PROP_SUBTYPE_ID, $docType);
			
			if ($docType == DOCTYPE_IMAGE)
			{
				// if saving an image, make sure that we store the dimensions
				//	of the image so we can display it appropriately later.
				if (function_exists ('getimagesize'))
				{
					list($width, $height, $type, $attr) = getimagesize ($path);
					$this->set(PROP_DATA, serialize (array ('width'=>$width,'height'=>$height)));
				}
				else
				{	
					$this->set(PROP_DATA, serialize(array ('width'=>0,'height'=>0)));			
				}
			}
			
			return true;
		}
		else
		{
			$this->addError (sprintf(str('STR_DOC_LOADFAILED'),$path), ERROR_TYPE_SERIOUS);
		}
	}
		
    public function getExternalLinkToDocument()
    {
        return "download.php?type={$this->getType()}&id={$this->get(PROP_ID)}";
    }
    
	/**
	 * This is the location of the file just after it was uploaded but
	 *	but before it is copied to its final location
	 *	@return string The full path to the file (with filename/extension)
	 */
	public function getTemporaryLocation ()
	{
		return $this->_temporaryPath;
	}         
    
    /**
    * @desc Retrieves the storage method object.
    */
    public function getDocumentStorage()
    {
        return storage();
    }
    
	/**
	 * This will copy the file from the temporary location to the 
	 *	final storage location deleting the temporary file 
	 * 	afterwards (if requested).
	 *	@param bool $deleteTemp true to delete the temp file after it has
	 *					has been copied.
	 *	@return bool Returns true if the file was copied, false otherwise.
	 */
	function storeFileToFinalLocation($deleteTemp=true)
	{	
		iwImage::clearThumbs($this);
		
        $storage = $this->getDocumentStorage();
        
		if ($storage->storeDocument($this))
		{
			if ($deleteTemp)
			{
				@unlink ($this->getTemporaryLocation());
			}
				
			return true;
		}
		else
		{
			$this->addError (sprintf(str('STR_DOC_STOREFAILED'),$pathFrom), ERROR_TYPE_SERIOUS);
			return false;
		}
	}
	
	/**
	 * This is a utility function to be used with image type documents
	 * 	that returns the image dimensions (stored in the database) in an
	 *	easy to read format.  If the height OR width is requested then
	 *	an integer is returned.  Otherwise, if height AND width is requested
	 * 	then an array is returned with the 'width' element holding the 
	 *	the width and the 'height' element holding the height of the image.
	 *	@param string One of (IMAGE_HEIGHT, IMAGE_WIDTH, IMAGE_BOTH)
	 *	@return mixed Either an int or an array (see description) 
	 */
	public function getImageDimensions($dimensionType /*IMAGE_HEIGHT,IMAGE_WIDTH,IMAGE_BOTH*/) 
	{
		if ($this->get(PROP_SUBTYPE_ID) == DOCTYPE_IMAGE)
		{
			$dimensions = unserialize ($this->get(PROP_DATA));			
			if (is_array($dimensions))
			{
				if ($dimensionType == IMAGE_HEIGHT)
				{				
					return $dimensions['height'];
				}
				else if ($dimensionType == IMAGE_WIDTH)
				{
					return $dimensions['width'];
				}
				else 
				{					
					return $dimensions;
				}
			}
		}		
		else
		{
			if ($dimensionType == IMAGE_BOTH)
			{
				return array ('width'=>0,'height'=>0);
			}
			else
			{
				return 0;
			}
		}		
	}
		
	/**
	 * This will send the document directly to the browser
	 *	handling the header information and anything else
	 *	that needs to be sent to make this a valid download.
	 *	That means that nothing can be output to the browser
	 *	before this.  Also, files that are marked as deleted
	 * 	cannot be downloaded unless an administrator is logged
	 *	in.
     * @param bool $logDownload If true, then the download attempt will be logged otherwise
     *          no log will be added causing the download count to
     *          remain as is.
	 */
	public function downloadItem ($logDownload)
	{
		if (!$this->get(PROP_STATUS)==ITEM_STATUS_ACTIVE &&
			!app()->isAdminLoggedIn())
		{
			// Access to this item is not allowed for this user.
			$content = 'Invalid Document Specified';
			header('Content-Type: text/plain');
			header('Content-Length: ' . strlen($content));
			header('Content-Disposition: attachment; filename=invalidfile.txt');			
			echo $content;			
			return;
		}
		
		// Get the content				
		$filename = $this->get(PROP_ORIGINAL_FILENAME);				

		$username = '';
		$user = app()->getLoggedInUserObject();
		if ($user != null)
		{
			$username = $user->get(PROP_USERNAME);
		}
        
		$filePath = storage()->retrieveDocument($this);
		if ($filePath)
		{
			if (($username != '') && $logDownload)
			{
				app()->notify (EVENT_OBJECT_DOWNLOADED,array('type'=>$this->getType(),'id'=>$this->getUniqueID(),'username'=>$username));				
			}
				
			// Replace spaces with '_' for downloading.
			$fileType = $this->getFileTypeObject();
            $mimeTypes = $fileType->getMimeType($filename);
            if (count($mimeTypes) > 0)
            {
                $mt = $mimeTypes[0];
            }
            else
            {
                $mt = 'application/octet-stream';
            }
                        
			$name = str_replace (' ', '_', $filename);
			header('Content-Description: File Transfer');
			header('Content-Type: '.$mt);
			header('Content-Length:'.filesize($filePath));
			header('Pragma: private');
			header('Cache-control: private, must-revalidate');
			header('Content-Disposition: attachment; filename='.$name);			
			readfile($filePath);
		}		
	}
	
	/**
	 * Displays the content of the file itself if possible.  This is in addition
	 *	to the document metadata.
	 */
	public function render ($params=false)
	{
		if (!$this->get(PROP_STATUS)==ITEM_STATUS_ACTIVE &&
			!app()->isAdminLoggedIn())
		{
			// Can't access this document but still log the attempt.
			//	
			app()->notify (EVENT_OBJECT_VIEWED,array('type'=>$this->getType(),'id'=>$this->getUniqueID(),'username'=>$username));
			
			// Return the error.
			return '<div class="error">'.STR('STR_DOC_INVALID_DOC_SPECIFIED').'</div>';
		}
		
		$html = '';

		$document = $this->getDetailedDocumentInfo();
		
		$comments = array();
		$comments['username'] = app()->getLoggedInUsername();
		$comments['item_id'] = $this->get(PROP_ID);
		$comments['post_date'] = iwUtilities::dateTimeFormat(time(),false);
		$comments['parent_post_id'] = 0;
		
		$document['actions'] = $this->getAvailableActions();
		
		if (app()->isUserLoggedIn())
		{
			$username = app()->getLoggedInUserObject()->get(PROP_USERNAME);
		}
		else
		{
			$username = '[unregistered]';
		}
		app()->notify (EVENT_OBJECT_VIEWED,array('type'=>$this->getType(),'id'=>$this->getUniqueID(),'username'=>$username));
        
        $headerInfo = '';
        nx_fire_event('EVENT_DOCUMENT_VIEW_SINGLE_HEADER',array('header'=>&$headerInfo,'doc'=>&$this));
        
        $document['additional_header_info'] = $headerInfo;
        
		$smarty = app()->getSmartyObject ();
		$smarty->assign ('document',$document);
		$smarty->assign ('post',$comments);
		return smarty_templ_fetch('views/document.tpl.html',$smarty);				
	}

	/**
	 * Override will delete the file on disk
	 */
	public function onAfterPermanentDelete ()
	{
		// Get rid of the file on disk as well as 
		//	the database record.	
		if (storage()->deleteDocument($this))
		{
            parent::onAfterPermanentDelete();			
		}
		else
		{
			$this->addError (sprintf(str('STR_DOC_DELETEFAILED_ID'),$this->get(PROP_ID)), ERROR_TYPE_SERIOUS);
		}
	}

	/**
	 * Gets all conversations for this document as shPost objects
     * @return array An array of shPost objects
	 */
	public function getConversation()
	{
		// CONVERSATIONS
		$conversations = new shConversations;
		return $conversations->getConversation($this->get(PROP_ID));
	}
    
    /**
    * @desc Gets storage filename for this object instance.
    */
    function getStorageFilename()
    {
        return '__filebox__'.sprintf ("%04d", $this->get(PROP_ID)).$extension;
    }    
}

?>
